package it.polito.bigdata.spark.example;

import org.apache.spark.api.java.*;

import scala.Tuple2;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.spark.SparkConf;

public class SparkDriver {

	public static void main(String[] args) {

		String inputPath;
		String inputPath2;
		Double threshold;
		String outputNameFileKML;

		inputPath = args[0];
		inputPath2 = args[1];
		threshold = new Double(args[2]);
		outputNameFileKML = args[3];

		// Create a configuration object and set the name of the application
		SparkConf conf = new SparkConf().setAppName("Spark Lab #7");

		// Create a Spark Context object
		JavaSparkContext sc = new JavaSparkContext(conf);

		// Read the content of the input file
		JavaRDD<String> inputRDD = sc.textFile(inputPath);

		// Remove the header and the lines with #free slots=0 && #used slots=0
		JavaRDD<String> filteredRDD = inputRDD.filter(new RemoveLines());

		// Map each line to a pair
		// key = StationId_DayOfTheWeek_Hour
		// value = (1,1) if the station is full, (1,0) if the station is not
		// full
		JavaPairRDD<String, CountTotReadingsTotFull> stationWeekDayHour = filteredRDD.mapToPair(new PairCounts());

		// Count the total number of readings and "full" readings for each key
		JavaPairRDD<String, CountTotReadingsTotFull> stationWeekDayHourCounts = stationWeekDayHour
				.reduceByKey(new Sum());
	
		// Compute criticality for each key
		JavaPairRDD<String, Double> stationWeekDayHourCriticality = stationWeekDayHourCounts
				.mapValues(new ComputeCriticality());

		// Select only the pairs with criticality > threshold
		JavaPairRDD<String, Double> selectedPairs = stationWeekDayHourCriticality.filter(new ApplyThr(threshold));

		
		// The next part of the code select for each station the timeslot (dayOfTheWeek_Hour) with 
		// the maximum cardinality. If there is more than one timeslot with the same criticality 
		// for the same station, only one of them is selected (see the problem specification). 
		
		// Create a new PairRDD with
		// key = station id
		// value = DayOfTheWeek - Hour - Criticality
		JavaPairRDD<String, DayOfWeekHourCrit> stationTimeslotCrit = selectedPairs
				.mapToPair(new PairStationOtherPart());

		// Select the timeslot (dayOfTheWeek_Hour) with the maximum criticality
		// for each station
		JavaPairRDD<String, DayOfWeekHourCrit> resultRDD = stationTimeslotCrit.reduceByKey(new Max());

		// Read the location of the stations 
		JavaPairRDD<String, String> stationLocation = sc.textFile(inputPath2).mapToPair(new StationLocation());

		// Join the locations with the "critical" stations
		JavaPairRDD<String, Tuple2<DayOfWeekHourCrit, String>> resultLocations = resultRDD.join(stationLocation);

		// Create a string containing the description of a marker, in the KML format, for each 
		// sensor and the associated information
		JavaRDD<String> resultKML = resultLocations.map(new ToKML());

		// There is at most one string for each station. We can use collect and
		// store the returned list in the main memory of the driver.
		List<String> localKML = resultKML.collect();

		// Store the result in one single file stored in the distributed file
		// system
		// Add header and footer, and the content of localKML in the middle
		Configuration confHadoop = new Configuration();

		try {
			URI uri = URI.create(outputNameFileKML);

			FileSystem file = FileSystem.get(uri, confHadoop);
			FSDataOutputStream outputFile = file.create(new Path(uri));

			BufferedWriter bOutFile = new BufferedWriter(new OutputStreamWriter(outputFile, "UTF-8"));

			// Header
			bOutFile.write("<kml xmlns=\"http://www.opengis.net/kml/2.2\"><Document>");
			bOutFile.newLine();

			// Markers
			for (String lineKML : localKML) {
				bOutFile.write(lineKML);
				bOutFile.newLine();
			}

			// Footer
			bOutFile.write("</Document></kml>");
			bOutFile.newLine();

			bOutFile.close();
			outputFile.close();

		} catch (IOException e1) {
			e1.printStackTrace();
		}

		// Close the Spark context
		sc.close();
	}
}
